%DSGE model pro odhad potencialniho produktu
clc;
close all;

%definice promennych
 %endogenni
 var 
    y,     //produkt
    r,     //nominalni urokova mira
    pi,    //inflace
    u,     //mezni uzitek ze spotreby
    d,     //poptavkovy sok
    n,     //mnostvi prace
    phi,   //mezni naklady
    z,     //nakladovy sok
    y_f,   //potencialni produkt
    a,     //technologicky pokrok
    u_f;   //mezni uzitek ze spotreby pri pruznych cenach 

 %exogenni
 varexo 
    epsilon_r,    //monetarni sok
    epsilon_d,    //poptavkovy sok
    epsilon_z,    //nakladovy sok
    epsilon_a;    //technologicky sok

 %parametry
 parameters 
    tau,        //inverze meziasov elasticity substituce 
    beta,       //diskontni faktor
    h,          //zvyk ve spotrebe
    omega,      //podil neoptimalizujicich firem
    eta,        //elasticita substituce mezi praci a spotrebou
    psi_y,      //elasticita urokove sazby na mezeru vstupu
    psi_pi,     //elasticita urokove sazby na inflaci
    rho_r,      //mira vyhlazeni urokove sazby
    rho_d,      //setrvacnost poptavkoveho soku
    rho_z,      //setrvacnost nabidkoveho soku
    rho_a;      //setrvacnost technologickeho pokroku

%kalibrace modelu - nastaveni hodnot paramteru
    tau = 2.3128;         
    beta = 0.99;      
    h = 0.9661;
    eta = 1.5442;         
    omega = 0.7944;               
    psi_pi = 1.3157;
    psi_y = 0.2706;      
    rho_r = 0.7290;     
    rho_d = 0.8308;     
    rho_z = 0.5724;     
    rho_a = 0.9336;

%modelove rovnice - rovnice jiz log-linearizovane - upresnime volbou linear
model(linear);
  u - y = u(+1) - y(+1) + r - pi(+1);
  u = 1/(1 - beta * h) * ((1 - tau) * ((1 + beta * h^2) * y - h * y(-1) - beta * h * y(+1)) + d - beta * h * d(+1));
  pi = beta * pi(+1) + (1 - beta * omega)*(1 - omega)/omega * phi + (1 - omega)/omega * (z - beta * omega * z(+1));
  y_f = a + 1/(1 + eta) * u_f - 1/(1 + eta) * d;
  r = rho_r * r(-1) + (1 - rho_r) * (psi_pi * pi + psi_y * (y - y_f)) + epsilon_r;
  d = rho_d * d(-1) + epsilon_d;
  z = rho_z * z(-1) + epsilon_z;
  a = rho_a * a(-1) + epsilon_a;
  u_f = 1/(1 - beta * h) * ((1 - tau) * ((1 + beta * h^2) * y_f - h * y_f(-1) - beta * h * y_f(+1)) + d - beta * h * d(+1));
  phi = d + eta * n - u + y - a; 
  y = n + a;
end;

%nastaveni pocatecnich podminek pro vypocet steady statu - model je gapovy, steady state je tedy pro vsechny promenne 0
initval;
    y = 0;    
    r = 0;     
    pi = 0;  
    d = 0;     
    n = 0;   
    z = 0;     
    y_f= 0;  
    a = 0;     
end;
%u, u_f, phi si sam dopocita

%specifikace soku
sigma_r = 0.3355;      %sigma_r - smerodatna odchylka pro monetarni sok
sigma_d = 1.0901;      %sigma_d - smerodatna odchylka pro poptavkovy sok
sigma_z = 0.4230;      %sigma_z - smerodatna odchylka pro nabidkovy sok
sigma_a = 1.4366;      %sigma_a - smerodatna odchylka pro technologicky sok

shocks;
var epsilon_r;
stderr  sigma_r;     
var epsilon_d;
stderr  sigma_d;     
var epsilon_z;
stderr  sigma_z;    
var epsilon_a;
stderr  sigma_a;    
end;

%vypocet steady statu
% vystup - lgy_ - abecedne serazene promenne, ys_ hodnoty steady statu
steady;

%jedina stabilni rovnovah v okoli steady statu - pocet vlastnich cisel modelu musi byt stejny jako pocet promenych v ocekavanich
%vystup - eigenvalues_ - hodnoty vlastnich cisel
check;

%stochasticka simulace - muzu za to napsat promenny pro ktery to chci
%volby -    irf - delka pro impulsni odezvy
%           simul - simulace modelu - data ulozena do y_ + kazda promena je ve svem nazvu
%           periods - delka simulace
stoch_simul(irf=20) y,pi,r,y_f;

save IRF_A pi_epsilon_a r_epsilon_a y_epsilon_a y_f_epsilon_a; 
save IRF_D pi_epsilon_d r_epsilon_d y_epsilon_d y_f_epsilon_d;
save IRF_R pi_epsilon_r r_epsilon_r y_epsilon_r y_f_epsilon_r;
save IRF_Z pi_epsilon_z r_epsilon_z y_epsilon_z y_f_epsilon_z;