%DSGE model pro odhad potencialniho produktu
clc;
close all;

%definice promennych
 %endogenni
 var 
    y,     //produkt
    r,     //nominalni urokova mira
    pi,    //inflace
    u,     //mezni uzitek ze spotreby
    d,     //poptavkovy sok
    n,     //mnostvi prace
    phi,   //mezni naklady
    z,     //nakladovy sok
    y_f,   //potencialni produkt
    a,     //technologicky pokrok
    u_f,   //mezni uzitek ze spotreby pri pruznych cenach
    gap,   //mezera vystupu
    y_poz, //hdp - pozorovane
    pi_poz,//inflace - pozorovane
    r_poz; //nominalni urokove mira - pozorovane

 %exogenni
 varexo 
    epsilon_r,    //monetarni sok
    epsilon_d,    //poptavkovy sok
    epsilon_z,    //nakladovy sok
    epsilon_a;    //technologicky sok

 %parametry
 parameters 
    tau,        //inverze meziasov elasticity substituce 
    beta,       //diskontni faktor
    h,          //zvyk ve spotrebe
    omega,      //podil neoptimalizujicich firem
    eta,        //elasticita substituce mezi praci a spotrebou
    psi_y,      //elasticita urokove sazby na mezeru vstupu
    psi_pi,     //elasticita urokove sazby na inflaci
    rho_r,      //mira vyhlazeni urokove sazby
    rho_d,      //setrvacnost poptavkoveho soku
    rho_z,      //setrvacnost nabidkoveho soku
    rho_a,      //setrvacnost technologickeho pokroku
    gamma_ss,   //steady-statovy rust technologickho soku
    pi_ss,      //steady-statova inflace
    r_ss;       //steady-statova realna urokova mira

%kalibrace modelu - nastaveni hodnot paramteru
    tau = 1.86;         
    %beta = 0.99;      
    h = 0.5;         
    omega = 0.66;      
    eta = 1;       
    psi_y = 0.5;      
    psi_pi = 1.5;    
    rho_r = 0.5;     
    rho_d = 0.5;     
    rho_z = 0.5;     
    rho_a = 0.5;
    gamma_ss = 0.5;   
    pi_ss = 3.36;      
    r_ss = 3; 
    beta = (exp(r_ss/400))^-1; %plyne z FOC ve steady-statu

%modelove rovnice - rovnice jiz log-linearizovane - upresnime volbou linear + pridame rovnice pro pozorovane veliciny
model(linear);
  u - y = u(+1) - y(+1) + r - pi(+1);
  u = 1/(1 - beta * h) * ((1 - tau) * ((1 + beta * h^2) * y - h * y(-1) - beta * h * y(+1)) + d - beta * h * d(+1));
  pi = beta * pi(+1) + (1 - beta * omega)*(1 - omega)/omega * phi + (1 - omega)/omega * (z - beta * omega * z(+1));
  y_f = a + 1/(1 + eta) * u_f - 1/(1 + eta) * d;
  r = rho_r * r(-1) + (1 - rho_r) * (psi_pi * pi + psi_y * (y - y_f)) + epsilon_r;
  d = rho_d * d(-1) + epsilon_d;
  z = rho_z * z(-1) + epsilon_z;
  a = rho_a * a(-1) + epsilon_a;
  u_f = 1/(1 - beta * h) * ((1 - tau) * ((1 + beta * h^2) * y_f - h * y_f(-1) - beta * h * y_f(+1)) + d - beta * h * d(+1));
  phi = d + eta * n - u + y - a; 
  y = n + a;
  y_poz = gamma_ss + y - y(-1);
  pi_poz = pi_ss + 4*pi;
  r_poz = r_ss + pi_ss + 4*r;
  gap = y - y_f;
end;

%nastaveni pocatecnich podminek pro vypocet steady statu - model je gapovy, steady state je tedy pro vsechny promenne 0
initval;
    y = 0;    
    r = 0;     
    pi = 0;  
    d = 0;     
    n = 0;   
    z = 0;     
    y_f= 0;  
    a = 0;     
    epsilon_r=0;
    epsilon_d=0;
    epsilon_z=0;
    epsilon_a=0;
end;
%u, u_f, phi, y_poz, pi_poz, r_poz, gap si sam dopocita

%specifikace soku
sigma_r = 0.5;      %sigma_r - smerodatna odchylka pro monetarni sok
sigma_d = 0.5;      %sigma_d - smerodatna odchylka pro poptavkovy sok
sigma_z = 0.5;      %sigma_z - smerodatna odchylka pro nabidkovy sok
sigma_a = 0.5;      %sigma_a - smerodatna odchylka pro technologicky sok

shocks;
var epsilon_r;
stderr  sigma_r;     
var epsilon_d;
stderr  sigma_d;     
var epsilon_z;
stderr  sigma_z;    
var epsilon_a;
stderr  sigma_a;    
end;

%vypocet steady statu
% vystup - lgy_ - abecedne serazene promenne, ys_ hodnoty steady statu
steady;

%jedina stabilni rovnovah v okoli steady statu - pocet vlastnich cisel modelu musi byt stejny jako pocet promenych v ocekavanich
%vystup - eigenvalues_ - hodnoty vlastnich cisel
check;

%odhadovane parametry a jejich priory
estimated_params;
tau,gamma_pdf,1.86,0.25;
h,beta_pdf,0.5,0.25;
omega,beta_pdf,0.66,0.08;
r_ss,gamma_pdf,3,0.4;
eta,gamma_pdf,1,0.4;
psi_pi,gamma_pdf,1.5,0.25;
psi_y,gamma_pdf,0.5,0.15;
rho_r,beta_pdf,0.5,0.25;
rho_d,beta_pdf,0.5,0.25;
rho_z,beta_pdf,0.5,0.25;
rho_a,beta_pdf,0.5,0.25;
gamma_ss,gamma_pdf,0.5,0.6;
pi_ss,gamma_pdf,3.36, 0.8;
stderr epsilon_r,inv_gamma_pdf, 0.5,0.3;
stderr epsilon_d,inv_gamma_pdf, 0.5,0.3;
stderr epsilon_z,inv_gamma_pdf, 0.5,0.3;
stderr epsilon_a,inv_gamma_pdf, 0.5,0.3;
end;

%upresneni pozorovanych velicin
varobs y_poz, pi_poz, r_poz;

%vytvorim si vektory pro TeXove soubory
global lgx_TeX_ lgy_TeX_ ;
lgx_TeX_ = '\epsilon_a'; 
lgx_TeX_ = strvcat(lgx_TeX_,'\epsilon_d'); 
lgx_TeX_ = strvcat(lgx_TeX_,'\epsilon_r');
lgx_TeX_ = strvcat(lgx_TeX_,'\epsilon_z'); 

lgy_TeX_ = 'a'; 
lgy_TeX_ = strvcat(lgy_TeX_,'d');
lgy_TeX_ = strvcat(lgy_TeX_,'gap');  
lgy_TeX_ = strvcat(lgy_TeX_,'n'); 
lgy_TeX_ = strvcat(lgy_TeX_,'\phi'); 
lgy_TeX_ = strvcat(lgy_TeX_,'\pi'); 
lgy_TeX_ = strvcat(lgy_TeX_,'\pi_{poz}');
lgy_TeX_ = strvcat(lgy_TeX_,'r');
lgy_TeX_ = strvcat(lgy_TeX_,'r_{poz}');
lgy_TeX_ = strvcat(lgy_TeX_,'u');
lgy_TeX_ = strvcat(lgy_TeX_,'u^f');
lgy_TeX_ = strvcat(lgy_TeX_,'y');
lgy_TeX_ = strvcat(lgy_TeX_,'y^f');
lgy_TeX_ = strvcat(lgy_TeX_,'y_{poz}');
lgy_TeX_ = strvcat(lgy_TeX_,'z');

estim_params_.tex = '\tau'; 
estim_params_.tex = strvcat(estim_params_.tex,'h'); 
estim_params_.tex = strvcat(estim_params_.tex,'\omega'); 
estim_params_.tex = strvcat(estim_params_.tex,'r^{ss}');
estim_params_.tex = strvcat(estim_params_.tex,'\eta');
estim_params_.tex = strvcat(estim_params_.tex,'\psi_{\pi}');
estim_params_.tex = strvcat(estim_params_.tex,'\psi_y');    
estim_params_.tex = strvcat(estim_params_.tex,'\rho_r'); 
estim_params_.tex = strvcat(estim_params_.tex,'\rho_d'); 
estim_params_.tex = strvcat(estim_params_.tex,'\rho_z'); 
estim_params_.tex = strvcat(estim_params_.tex,'\rho_a');
estim_params_.tex = strvcat(estim_params_.tex,'\gamma^{ss}');
estim_params_.tex = strvcat(estim_params_.tex,'\pi^{ss}');

options_.varobs_TeX = 'y_{poz}'; 
options_.varobs_TeX = strvcat(options_.varobs_TeX,'\pi_{poz}'); 
options_.varobs_TeX = strvcat(options_.varobs_TeX,'r_{poz}');

%odhad
%volby - conf_sig - hladina pro vypocet intervalu spolehlivosti (0.9)
%        mh_replic - pocet replikaci v M-H algoritmu (20000)
%        mh_nblocks - pocet paralelnich bloku v M-H (2)
%        mh_drop - podil vyhozenych vzorku (0.5)
%        mh_jscale - velikost skoku pri hledani v M-H - ovlivnuje velikost akceptacni pravdepodobnosti (0.2)
%        bayesian_irf - bayesianksa IRF, delka dana irf
%        smoother - pomoci KF vypocita smoothovane hodnoty parametru
%        forecast - predpoved
%        tex - vyhazuje soubory pro TeX

estimation(datafile=data_source,mh_replic=100000,mh_jscale=0.35,smoother, tex);